import http from '@/support/http';
import { Code } from '@/enum/app';
import Message from '@/support/message';
import { ref, watch } from 'vue';
import { isFunction } from '@/support/helper';

export function useDestroy(type = 'delete', confirm = '确认删除吗') {
  const isDeleted = ref(false);

  const beforeDestroy = ref();

  // fetch list
  function destroy(path: string, id: string | number) {
    Message.confirm(confirm + '?', function () {
      // before destroy
      if (isFunction(beforeDestroy.value)) {
        beforeDestroy.value();
      }

      http[type](path + '/' + id)
        .then(r => {
          if (r.data.code === Code.SUCCESS) {
            Message.success(r.data.message);
            isDeleted.value = true;
          } else {
            Message.error(r.data.message);
          }
        })
        .finally(() => {});
    });
  }

  const deleted = (reset: Function) => {
    watch(isDeleted, function (value) {
      if (value) {
        isDeleted.value = false;
        reset();
      }
    });
  };

  return { destroy, deleted };
}
